@@ -47,6 +47,7 @@ class Algorithm():
             Severity level for event logging.  If the value is None,
             log_level is set to the value of skrt.core.Defaults().log_level.
         '''
+        self.opts = opts
         class_name = type(self).__name__
         self.name = class_name if name is None else name
         self.log_level = \
@@ -142,7 +143,7 @@ class Application():
         '''
         for alg in self.algs:
             self.status = alg.execute(patient=patient)
-            if not self.status.OK():
+            if not self.status.ok():
                 if not self.status.reason:
                     self.status.reason = (
                         f'Problem executing algorithm "{alg.name}" for data '
@@ -157,7 +158,7 @@ class Application():
         '''
         for alg in self.algs:
             self.status = alg.finalise()
-            if not self.status.OK():
+            if not self.status.ok():
                 if not self.status.reason:
                     self.status.reason = \
                         f'Problem finalising algorithm "{alg.name}"'
@@ -171,7 +172,7 @@ class Application():
         '''
 
         for alg in self.algs:
-            if not self.status.OK():
+            if not self.status.ok():
                 if not self.status.reason:
                     self.status.reason = \
                         f'Problem initialising algorithm "{alg.name}"'
@@ -189,16 +190,16 @@ class Application():
             List of paths to folders containing patient data.
         '''
 
-        if self.status.OK():
+        if self.status.ok():
             if not paths:
                 self.logger.warning('List of paths to patient data is empty')
             for dataPath in paths:
                 patient = Patient(path=dataPath)
                 self.status = self.execute(patient=patient)
-                if not self.status.OK():
+                if not self.status.ok():
                     break
 
-        if self.status.OK():
+        if self.status.ok():
             self.status = self.finalise()
 
         return self.status
@@ -235,13 +236,13 @@ class Status():
         self.reason = reason
         return None
 
-    def OK(self):
+    def ok(self):
         '''
         Return boolean indicating whether status is okay (exit code non-zero).
         '''
         if self.code:
-            isOK = False
+            is_ok = False
         else:
-            isOK = True
+            is_ok = True
 
-        return isOK
+        return is_ok
