@@ -5,6 +5,7 @@ import functools
 import numpy as np
 import os
 import time
+import pydicom
 from logging import getLogger, Formatter, StreamHandler
 from typing import Any, List, Optional, Tuple
 
@@ -92,6 +93,10 @@ Defaults({"print_depth": 0})
 # 'WARNING' (30), 'ERROR' (40), 'CRITICAL' (50)
 Defaults({"log_level": "WARNING"})
 
+# Lengths of date and time stamps.
+Defaults({"len_date": 8})
+Defaults({"len_time": 6})
+
 
 class Data:
     """
@@ -205,6 +210,10 @@ class Data:
                 else:
                     value_string = "{}"
 
+            # Handle printing of pydicom datasets
+            elif isinstance(item, pydicom.dataset.FileDataset):
+                value_string = str(item.__class__)
+
             # Handle printing of nested Data objects
             else:
                 if issubclass(item.__class__, Data):
@@ -219,39 +228,106 @@ class Data:
         out.append("}")
         return "\n".join(out)
 
-    def clone(self):
+    def clone(self, data_types_to_copy=None, copy_data=True):
         """
-        Return a clone of the Data object containing copies of any 
-        lists, dicts, or arrays owned by the object.
+        Return a clone of the Data object. All attributes of the original
+        object will be copied by reference to the new object, with some 
+        exceptions (see parameters below).
+
+
+        Parameters
+        ----------
+        data_types_to_copy : list, default=None
+            List of types inherting from the Data class.
+            Any objects of the types in this list that are either directly 
+            stored as an attribute or stored in a list or dict attribute will 
+            be cloned, rather than assigning the same object as to an 
+            attribute of the cloned parent object.
+            (Note that these child Data object will be copied with 
+            data_types_to_copy=None to prevent recursion.)
+
+        copy_data : bool, default=True
+            If True, any lists, dicts, and numpy arrays will be shallow 
+            copied rather than copied by reference.
         """
 
         clone = copy.copy(self)
-        self.clone_attrs(clone)
+        self.clone_attrs(clone, data_types_to_copy, copy_data)
         return clone
 
-    def clone_attrs(self, obj):
+    def clone_attrs(self, obj, data_types_to_copy=None, copy_data=True):
         """
-        Apply class attributes from self to a new object, making copies 
-        of any lists, dicts, or numpy arrays.
+        Assign all attributes of <self> to another object, <obj>.
+
+
+        Parameters
+        ----------
+        obj : object
+            Object to which attributes of <self> will be copied.
+
+        data_types_to_copy : list, default=None
+            List of types inherting from the Data class.
+            Any objects of the types in this list that are either directly 
+            stored as an attribute or stored in a list or dict attribute will 
+            be cloned, rather than assigning the same object as to an 
+            attribute of the cloned parent object.
+            (Note that these child Data object will be copied with 
+            data_types_to_copy=None to prevent recursion.)
+
+        copy_data : bool, default=True
+            If True, any lists, dicts, and numpy arrays will be shallow 
+            copied rather than copied by reference.
         """
 
-        for attr in dir(self):
+        # Check the data types to copy are valid
+        dtypes_valid = []
+        if data_types_to_copy is not None:
+            for dtype in data_types_to_copy:
+                if issubclass(dtype, Data):
+                    dtypes_valid.append(dtype)
+                else:
+                    print("Warning: data_types_to_copy must inherit from "
+                          "skrt.Data! Type", dtype, "will be ignored.")
+
+        for attr_name in dir(self):
 
             # Don't copy private variables
-            if attr.startswith("__"):
+            if attr_name.startswith("__"):
                 continue
 
-            # Don't copy methods
-            if callable(getattr(self, attr)):
+            # Don't copy methodms
+            attr = getattr(self, attr_name)
+            if callable(attr):
                 continue
             
             # Make new copy of lists/dicts/arrays
-            if type(getattr(self, attr)) in [list, dict, np.ndarray]:
-                setattr(obj, attr, copy.copy(getattr(self, attr)))
+            if copy_data and type(attr) in [dict, list, np.ndarray]:
+                setattr(obj, attr_name, copy.copy(attr))
+
+                # Also clone given Data types
+                if isinstance(attr, list):
+                    for i, item in enumerate(attr):
+                        for dtype in dtypes_valid:
+                            if isinstance(item, dtype):
+                                getattr(obj, attr_name)[i] = item.clone() 
+                                break
+                elif isinstance(attr, dict):
+                    for key, item in attr.items():
+                        for dtype in dtypes_valid:
+                            if isinstance(item, dtype):
+                                getattr(obj, attr_name)[key] = item.clone() 
+                                break
+
+            # Clone any owned Data objects if cloning children
+            elif issubclass(type(attr), Data):
+                for dtype in dtypes_valid:
+                    if isinstance(attr, dtype):
+                        setattr(obj, attr_name, attr.clone())
+                        break
 
             # Otherwise, copy reference to attribute
             else:
-                setattr(obj, attr, getattr(self, attr))
+                setattr(obj, attr_name, attr)
 
     def get_dict(self) -> dict:
         """
@@ -374,7 +450,7 @@ class Dated(PathData):
             try:
                 self.date, self.time = timestamp.split("_")
             except ValueError:
-                self.date, self.time = (None, None)
+                self.date, self.time = ("", "")
 
         # Set date and time from current time
         if not self.date and auto_timestamp:
@@ -382,7 +458,10 @@ class Dated(PathData):
             self.time = time.strftime("%H%M%S")
 
         # Make full timestamp string
-        self.timestamp = f"{self.date}_{self.time}"
+        if not self.date and not self.time:
+            self.timestamp = ""
+        else:
+            self.timestamp = f"{self.date}_{self.time}"
 
     def in_date_interval(self, 
                          min_date: Optional[str] = None, 
@@ -578,6 +657,14 @@ def is_timestamp(string: str = "") -> bool:
             if not item.isdigit():
                 valid = False
                 break
+        if valid:
+            if type(Defaults().len_date) == int:
+                if len(items[0]) != Defaults().len_date:
+                    valid = False
+            if type(Defaults().len_time) == int:
+                if len(items[1]) != Defaults().len_time:
+                    valid = False
+
     return valid
 
 
@@ -591,17 +678,17 @@ def is_list(var: Any) -> bool:
     return is_a_list
 
 
-def to_three(val: Any) -> Optional[List]:
-    """Ensure that a value is a list of three items."""
+def to_list(val: Any, n : int = 3, keep_none_single : bool = True) -> Optional[List]:
+    """Ensure that a value is a list of n items."""
 
-    if val is None:
+    if val is None and keep_none_single:
         return None
     if is_list(val):
-        if not len(val) == 3:
-            print(f"Warning: {val} should be a list containing 3 items!")
+        if not len(val) == n:
+            print(f"Warning: {val} should be a list containing {n} items!")
         return list(val)
     elif not is_list(val):
-        return [val, val, val]
+        return [val] * n
 
 
 def generate_timestamp() -> str:
