@@ -81,7 +81,7 @@ class Study(skrt.core.Archive):
         images.append(im)
 
         # Ensure image has a timestamp
-        if im.timestamp == "None_None":
+        if not im.timestamp:
             im.timestamp = skrt.core.generate_timestamp()
 
         # Ensure corresponding structure list exists
@@ -94,7 +94,7 @@ class Study(skrt.core.Archive):
         structure_sets = getattr(self, struct_name)
         for structure_set in im.get_structure_sets():
             structure_sets.append(structure_set)
-            if structure_set.timestamp == "None_None":
+            if not structure_set.timestamp:
                 structure_set.timestamp = skrt.core.generate_timestamp()
 
     def correct_dose_image_position(self, doses=[]):
@@ -341,11 +341,17 @@ class Study(skrt.core.Archive):
             for file in group.files:
 
                 # Create StructureSet
-                structure_set = StructureSet(file.path, image=image)
+                structure_set = StructureSet(
+                    file.path, 
+                    image=image,
+                    name=None,
+                    load=False
+                )
 
                 # Add to Image
                 if image is not None:
                     image.add_structure_set(structure_set)
+                    structure_set.name = f'Structure set {len(image.structure_sets)}'
 
                 # Add to list of all structure sets
                 structure_sets.append(structure_set)
@@ -363,7 +369,7 @@ class Study(skrt.core.Archive):
         if obj:
             if obj.files:
                 image_path = obj.files[-1].path
-                ds = pydicom.read_file(fp=image_path, force=True)
+                ds = pydicom.dcmread(fp=image_path, force=True)
                 if hasattr(ds, "StudyDescription"):
                     description = ds.StudyDescription
 
@@ -398,7 +404,7 @@ class Patient(skrt.core.PathData):
     """Object associated with a top-level directory whose name corresponds to
     a patient ID, and whose subdirectories contain studies."""
 
-    def __init__(self, path=None, exclude=["logfiles"]):
+    def __init__(self, path="", exclude=["logfiles"]):
 
         # Set path and patient ID
         if path is None:
@@ -496,7 +502,7 @@ class Patient(skrt.core.PathData):
 
         # Read demographic info from the object
         if obj and obj.files:
-            ds = pydicom.read_file(fp=obj.files[-1].path, force=True)
+            ds = pydicom.dcmread(fp=obj.files[-1].path, force=True)
             for key in info:
                 for prefix in ["Patient", "Patients"]:
                     attr = f"{prefix}{key[0].upper()}{key[1:]}"
