@@ -53,9 +53,9 @@ class SyntheticImage(Image):
         """
 
         # Create image properties
-        shape = skrt.core.to_three(shape)
+        shape = skrt.core.to_list(shape)
         self.shape = [shape[1], shape[0], shape[2]]
-        self.input_voxel_size = [abs(v) for v in skrt.core.to_three(voxel_size)]
+        self.input_voxel_size = [abs(v) for v in skrt.core.to_list(voxel_size)]
         self.input_origin = origin
         self.max_hu = 0 if noise_std is None else noise_std * 3
         self.min_hu = -self.max_hu if self.max_hu != 0 else -20
@@ -63,7 +63,8 @@ class SyntheticImage(Image):
         self.bg_intensity = intensity
         self.background = self.make_background()
         self.shapes = []
-        self.roi_shapes = []
+        self.roi_shapes = {}
+        self.rois = {}
         self.groups = {}
         self.shape_count = {}
         self.translation = None
@@ -79,21 +80,21 @@ class SyntheticImage(Image):
             self.filename = os.path.expanduser(filename)
             self.write()
 
-    def view(self, **kwargs):
-        """View with QuickViewer."""
+    #  def view(self, **kwargs):
+        #  """View with QuickViewer."""
 
-        from skrt.viewer import QuickViewer
+        #  from skrt.viewer import QuickViewer
 
-        qv_kwargs = {
-            "hu": [self.min_hu, self.max_hu],
-            "title": "",
-            "origin": self.origin,
-            "voxel_size": self.voxel_size,
-            "mpl_kwargs": {"interpolation": "none"},
-        }
-        qv_kwargs.update(kwargs)
-        rois = self.get_roi_data()
-        QuickViewer(self.get_data(), structs=rois, **qv_kwargs)
+        #  qv_kwargs = {
+            #  "hu": [self.min_hu, self.max_hu],
+            #  "title": "",
+            #  "origin": self.origin,
+            #  "voxel_size": self.voxel_size,
+            #  "mpl_kwargs": {"interpolation": "none"},
+        #  }
+        #  qv_kwargs.update(kwargs)
+        #  rois = self.get_roi_data()
+        #  QuickViewer(self.get_data(), structs=rois, **qv_kwargs)
 
     def get_data(self):
         """Get data with noise overlaid."""
@@ -114,40 +115,55 @@ class SyntheticImage(Image):
 
         self.sdata = self.get_data()
         self.structure_sets = [self.get_structure_set()]
-        Image.plot(self, structure_set=0, *args, **kwargs)
+        Image.plot(self, rois=0, *args, **kwargs)
 
     def get_roi_data(self):
         """Get dict of ROIs and names, with any transformations applied."""
 
         roi_data = {}
-        for shape in self.roi_shapes:
+        for name, shape in self.roi_shapes.items():
             data = shape.get_data(self.get_coords())
-            roi_data[shape.name] = data
+            roi_data[name] = data
         return roi_data
 
     def get_structure_set(self):
         """Make StructureSet object of own structures."""
 
-        structure_set = StructureSet()
-        for shape in self.roi_shapes:
-            structure_set.add_roi(
-                shape.get_data(self.get_coords()), name=shape.name, affine=self.affine
-            )
+        structure_set = StructureSet(name="StructureSet")
+        self.update_rois()
+        for roi in self.rois.values():
+            structure_set.add_roi(roi)
         return structure_set
 
+    def update_roi(self, name):
+        """Update an ROI to ensure it has the correct data."""
+        
+        self.rois[name].data = self.roi_shapes[name].get_data(self.get_coords())
+
+    def update_rois(self):
+        """Update all ROIs to have the correct data."""
+
+        for name in self.rois:
+            self.update_roi(name)
+
     def get_structure_sets(self):
         return [self.get_structure_set()]
 
     def get_roi(self, name):
         """Get a named ROI as an ROI object."""
 
-        roi_dict = {s.name: s for s in self.roi_shapes}
-        if name not in roi_dict:
+        if name not in self.rois:
             print("ROI", name, "not found!")
             return
 
-        s = roi_dict[name]
-        return ROI(s.get_data(self.get_coords()), name=name, affine=self.affine)
+        self.update_roi(name)
+        return self.rois[name]
+
+    def get_rois(self):
+        """Get list of all owned ROI objects."""
+
+        self.update_rois()
+        return list(self.rois.values())
 
     def write(self, outname=None, overwrite_roi_dir=False):
         """Write image data to an output file."""
@@ -206,19 +222,28 @@ class SyntheticImage(Image):
             if group is not None:
                 if group not in self.groups:
                     self.groups[group] = ShapeGroup([shape], name=group)
-                    self.roi_shapes.append(self.groups[group])
+                    self.roi_shapes[group] = self.groups[group]
+                    self.rois[group] = ROI(
+                        shape.get_data(self.get_coords()), name=group, 
+                        affine=self.affine
+                    )
                 else:
                     self.groups[group].add_shape(shape)
             else:
-                self.roi_shapes.append(shape)
 
-        if shape_type not in self.shape_count:
-            self.shape_count[shape_type] = 1
-        else:
-            self.shape_count[shape_type] += 1
+                if shape_type not in self.shape_count:
+                    self.shape_count[shape_type] = 1
+                else:
+                    self.shape_count[shape_type] += 1
 
-        if shape.name is None:
-            shape.name = f"{shape_type}{self.shape_count[shape_type]}"
+                if shape.name is None:
+                    shape.name = f"{shape_type}{self.shape_count[shape_type]}"
+
+                self.roi_shapes[shape.name] = shape
+                self.rois[shape.name] = ROI(
+                    shape.get_data(self.get_coords()), name=shape.name, 
+                    affine=self.affine
+                )
 
         self.min_hu = min([shape.intensity, self.min_hu])
         self.max_hu = max([shape.intensity, self.max_hu])
@@ -235,7 +260,7 @@ class SyntheticImage(Image):
     ):
 
         if centre is None:
-            centre = self.get_image_centre()
+            centre = self.get_centre()
         sphere = Sphere(self.shape, radius, centre, intensity, name)
         self.add_shape(sphere, "sphere", is_roi, above, group)
 
@@ -253,7 +278,7 @@ class SyntheticImage(Image):
     ):
 
         if centre is None:
-            centre = self.get_image_centre()
+            centre = self.get_centre()
         cylinder = Cylinder(self.shape, radius, length, axis, centre, intensity, name)
         self.add_shape(cylinder, "cylinder", is_roi, above, group)
 
@@ -284,8 +309,8 @@ class SyntheticImage(Image):
     ):
 
         if centre is None:
-            centre = self.get_image_centre()
-        side_length = skrt.core.to_three(side_length)
+            centre = self.get_centre()
+        side_length = skrt.core.to_list(side_length)
 
         cuboid = Cuboid(self.shape, side_length, centre, intensity, name)
         self.add_shape(cuboid, "cuboid", is_roi, above, group)
@@ -328,7 +353,7 @@ class SyntheticImage(Image):
                         get_translation_matrix(*[-d for d in self.translation])
                     )
                 if self.rotation:
-                    centre = self.get_image_centre()
+                    centre = self.get_centre()
                     transform = transform.dot(
                         get_rotation_matrix(*[-r for r in self.rotation], centre)
                     )
@@ -417,7 +442,7 @@ class Cuboid:
     def __init__(self, shape, side_length, centre, intensity, name=None):
 
         self.name = name
-        self.side_length = skrt.core.to_three(side_length)
+        self.side_length = skrt.core.to_list(side_length)
         self.centre = centre
         self.intensity = intensity
 
@@ -472,8 +497,8 @@ class Grid:
     def __init__(self, shape, spacing, thickness, intensity, axis=None, name=None):
 
         self.name = name
-        self.spacing = skrt.core.to_three(spacing)
-        self.thickness = skrt.core.to_three(thickness)
+        self.spacing = skrt.core.to_list(spacing)
+        self.thickness = skrt.core.to_list(thickness)
         self.intensity = intensity
         self.axis = axis
         self.shape = shape
